@@ -28,12 +28,11 @@
 #'  (being downloaded or already existing).
 #'
 #' @author Luigi Ranghetti, phD (2020) \email{luigi@@ranghetti.info}
-#' @author Lorenzo Busetto, phD (2019) \email{lbusett@@gmail.com}
+#' @author Lorenzo Busetto, phD (2019)
 #' @references L. Ranghetti, M. Boschetti, F. Nutini, L. Busetto (2020).
 #'  "sen2r": An R toolbox for automatically downloading and preprocessing 
-#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. DOI: 
-#'  \href{https://doi.org/10.1016/j.cageo.2020.104473}{10.1016/j.cageo.2020.104473}, 
-#'  URL: \url{http://sen2r.ranghetti.info/}.
+#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. 
+#'  \doi{10.1016/j.cageo.2020.104473}, URL: \url{http://sen2r.ranghetti.info/}.
 #' @note License: GPL 3.0
 #' @importFrom httr RETRY authenticate progress write_disk
 #' @importFrom foreach foreach "%do%"
@@ -120,6 +119,9 @@ s2_download <- function(
   s2_prodlist <- as(s2_prodlist, "safelist")
   # TODO add input checks
   s2_meta <- safe_getMetadata(s2_prodlist, info = "nameinfo")
+  # if (!is.null(attr(s2_prodlist, "footprint"))) {
+  #   s2_meta[,footprint:=attr(s2_prodlist, "footprint")]
+  # }
   
   # check input server
   s2_server <- ifelse(
@@ -232,6 +234,15 @@ s2_download <- function(
   # to avoid NOTE on check
   i <- mission <- level <- sensing_datetime <- id_orbit <- id_tile <- NULL
   
+  # Check connection
+  if (!check_scihub_connection()) {
+    print_message(
+      type = "error", 
+      "Impossible to reach the SciHub server ",
+      "(internet connection or SciHub may be down)." 
+    )
+  }
+  
   # read credentials
   if (length(s2_prodlist) > 0) {
     creds <- read_scihub_login(apihub)
@@ -261,15 +272,25 @@ s2_download <- function(
     zip_path <- file.path(outdir, paste0(names(s2_prodlist[i]),".zip"))
     safe_path <- gsub("\\.zip$", "", zip_path)
     
-    # regular expression to detect if equivalent products already exist
-    safe_regex <- s2_meta[i,paste0(
-      "^S",mission,"\\_MSIL",level,"\\_",strftime(sensing_datetime,"%Y%m%dT%H%M%S"),
-      "\\_N[0-9]{4}\\_R",id_orbit,"\\_T",id_tile,"\\_[0-9]{8}T[0-9]{6}\\.SAFE$"
-    )]
-    safe_existing <- list.files(dirname(zip_path), safe_regex, full.names = TRUE)
+    # # regular expression to detect if equivalent products already exist
+    # safe_regex <- s2_meta[i,paste0(
+    #   "^S",mission,"\\_MSIL",level,"\\_",strftime(sensing_datetime,"%Y%m%dT%H%M%S"),
+    #   "\\_N[0-9]{4}\\_R",id_orbit,"\\_T",id_tile,"\\_[0-9]{8}T[0-9]{6}\\.SAFE$"
+    # )]
+    # safe_existing <- list.files(dirname(zip_path), safe_regex, full.names = TRUE)
+    # safe_existing <- safe_existing[safe_isvalid(safe_existing)]
+    # 
+    # # if footprint exists, check if existing SAFEs are actually equivalent
+    # if (!is.null(s2_meta$footprint)) {
+    #   safe_existing_footprint <- safe_getMetadata(safe_existing, "footprint")
+    #   safe_existing_centroid <- st_centroid(st_transform(st_as_sfc(safe_existing_footprint, crs = 4326), 3857))
+    #   safe_centroid <- st_centroid(st_transform(st_as_sfc(s2_meta[i,footprint], crs = 4326), 3857))
+    #   centroid_distance <- st_distance(safe_existing_centroid, safe_centroid)[1,1]
+    #   # TODO
+    # }
+    
+    if (any(overwrite == TRUE, !dir.exists(safe_path))) {
     
-    if (any(overwrite == TRUE, length(safe_existing) == 0)) {
-      
       print_message(
         type = "message",
         date = TRUE,
@@ -335,8 +356,8 @@ s2_download <- function(
         suppressWarnings(file.remove(paste0(zip_path,".aria2")))
         print_message(
           type = "error",
-          "Download of file", link, "failed more than 10 times. ",
-          "Internet connection or SciHub may be down."
+          "Download of file", link, "failed more than 10 times ",
+          "(internet connection or SciHub may be down)."
         )
       } else {
         # check md5
@@ -383,7 +404,7 @@ s2_download <- function(
       }
       
     } else {
-      
+
       print_message(
         type = "message",
         date = TRUE,
@@ -391,11 +412,13 @@ s2_download <- function(
         " of ",length(s2_prodlist)," ",
         "since the corresponding folder already exists."
       )
+
       
-      safe_existing_meta <- safe_getMetadata(safe_existing, info = "nameinfo")
-      safe_newname <- safe_existing_meta$name[
-        order(nn(safe_existing_meta$creation_datetime), decreasing = TRUE)[1]
-        ]
+      # safe_existing_meta <- safe_getMetadata(safe_existing, info = "nameinfo")
+      # safe_newname <- safe_existing_meta$name[
+      #   order(nn(safe_existing_meta$creation_datetime), decreasing = TRUE)[1]
+      #   ]
+      safe_newname <- basename(safe_path)
       
     }
     
