@@ -16,12 +16,11 @@
 #'  available for download, FALSE if it is not or NA in case of errors with
 #'  the SAFE url.
 #' @author Luigi Ranghetti, phD (2019) \email{luigi@@ranghetti.info}
-#' @author Lorenzo Busetto, phD (2020) \email{lbusett@@gmail.com}
+#' @author Lorenzo Busetto, phD (2020)
 #' @references L. Ranghetti, M. Boschetti, F. Nutini, L. Busetto (2020).
 #'  "sen2r": An R toolbox for automatically downloading and preprocessing 
-#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. DOI: 
-#'  \href{https://doi.org/10.1016/j.cageo.2020.104473}{10.1016/j.cageo.2020.104473}, 
-#'  URL: \url{http://sen2r.ranghetti.info/}.
+#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. 
+#'  \doi{10.1016/j.cageo.2020.104473}, URL: \url{http://sen2r.ranghetti.info/}.
 #' @note License: GPL 3.0
 #' @importFrom httr RETRY authenticate content
 #' @importFrom jsonlite fromJSON
@@ -75,8 +74,9 @@ safe_is_online <- function(s2_prodlist = NULL, apihub = NA, verbose = TRUE) {
       ), as = "parsed", encoding = "UTF-8")), 
       error = function(e) {
         print_message(
-          type = "warning",
-          "Some error occurred (the Copernicus API Hub may be down)."
+          type = "error",
+          "Impossible to reach the SciHub server ",
+          "(internet connection or SciHub may be down)." 
         )
         NA
       }
