@@ -13,9 +13,8 @@
 #' @author Luigi Ranghetti, phD (2019) \email{luigi@@ranghetti.info}
 #' @references L. Ranghetti, M. Boschetti, F. Nutini, L. Busetto (2020).
 #'  "sen2r": An R toolbox for automatically downloading and preprocessing 
-#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. DOI: 
-#'  \href{https://doi.org/10.1016/j.cageo.2020.104473}{10.1016/j.cageo.2020.104473}, 
-#'  URL: \url{http://sen2r.ranghetti.info/}.
+#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. 
+#'  \doi{10.1016/j.cageo.2020.104473}, URL: \url{http://sen2r.ranghetti.info/}.
 #' @note License: GPL 3.0
 #' @export
 #' @import data.table
@@ -72,7 +71,7 @@ sen2r_getElements <- function(s2_names, format="data.table", abort=TRUE) {
     metadata$type <- ifelse(
       !grepl(fs2nc_regex$regex,s2_names), "unrecognised",
       ifelse(
-        grepl("[0-9]{2}[A-Z]{3}",metadata$extent_name), "tile",
+        grepl("[0-9]{2}[A-Z]{3}[a-z]?",metadata$extent_name), "tile",
         ifelse(
           metadata$extent_name=="", "merged", "clipped"
         )
